/*	radio.go - M.A.R-T>Y. notification daemon written in Go
	Copyright (C) 2023-2024 hoff.industries

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>. */

package main

import (
	"fmt"
	"os"

	//"github.com/romychs/gotk3/glib"
	"github.com/romychs/gotk3/libnotify"
	"hoff.industries/marty/libmarty"
	"hoff.industries/marty/libmmp"
	"hoff.industries/marty/mendpoints"
)

var TARGET_PROTOCOL = []int{2, 0, 0}
var INSTANCE_VERSION = []int{3, 0, 0}

const INSTANCE_NAME string = "m-goradio"
const INSTANCE_TITLE string = "Push Notification Daemon"

var Endpoints = []libmarty.MartyClientEndpoint{

	{
		Endpoint: mendpoints.RADIO_EPT_LOW,
		Name: "low",
		Min: 6,
		Max: 7,
		IsAscii: true,
	},

	{
		Endpoint: mendpoints.RADIO_EPT_NORMAL,
		Name: "normal",
		Min: 6,
		Max: 7,
		IsAscii: true,
	},

	{
		Endpoint: mendpoints.RADIO_EPT_CRITICAL,
		Name: "critical",
		Min: 6,
		Max: 7,
		IsAscii: true,
	},
}

var MaxEndpoint = len(Endpoints)

var NOTIFICATION_MAP = []libnotify.NotifyUrgency{
	libnotify.NOTIFY_URGENCY_LOW,
	libnotify.NOTIFY_URGENCY_NORMAL,
	libnotify.NOTIFY_URGENCY_CRITICAL,
}

func do_notify(title string, message string, urgency libnotify.NotifyUrgency) {

	var formatTitle string = fmt.Sprintf("%s %s", libmarty.BracketFlavour, title)

	pushNotification, err := libnotify.NotifyNotificationNew(formatTitle, message, "marty")

	if (err != nil) {
		return
	}

	//entryHint, err := glib.VariantStringNew(INSTANCE_NAME)

	//if (err == nil) {
	//}

	pushNotification.SetUrgency(urgency)
	pushNotification.Show()

	pushNotification = nil
}

func main() {

	var data libmarty.MartyClientData
	var status libmmp.MMStatus

	var title string = ""
	var message string = ""

	var titleIndex = -1
	var messageIndex = -1

	var txPacket libmmp.MMPacket
	var rxPacket *libmmp.MMPacket = nil

	var applicationStatus = libmmp.MMPArgs[libmmp.MMP_TYPE_APPLICATION]["SUCCESS"]

	var hostname string = ""
	var callError error = nil

	var endpointConversion int = -1
	var endpoint *libmarty.MartyClientEndpoint = nil

	var deliveryHandle *libmarty.MartyMessageHandle = nil

	libmarty.Marty_client_data_factory(&data)

	libmmp.Mmstatus_factory(&status)
	libmmp.Mmpacket_factory(&txPacket, true)

	data.VersionString = fmt.Sprintf("%s v%d.%d.%d", INSTANCE_NAME,
		INSTANCE_VERSION[0], INSTANCE_VERSION[1], INSTANCE_VERSION[2])

	fmt.Printf("%s[%s]%s Starting %s %s - %s\n",
		libmarty.LoggerColours[libmarty.LOG_INFO], libmarty.MartyFlavour,
			libmarty.ClearColour, libmarty.LIBMARTY_AUTHOR, INSTANCE_TITLE, data.VersionString)

	hostname, callError = os.Hostname()
	if (callError != nil) {

		os.Exit(libmarty.EXIT_ERR_HOSTNAME)
	}

	//data.TargetSocket = libmarty.TestSocket
	data.ConnType = libmarty.CONN_UNIX

	data.InitParameters.Auth = ""
	data.InitParameters.Priority = 0
	data.InitParameters.Host = hostname
	data.InitParameters.Name = "radio"

	callError = libnotify.Init(data.VersionString)
	if (callError != nil) {

		libmarty.Marty_full_log("Libnotify initialisation failure!\n",
			libmarty.LOG_CRITICAL, &data)
		os.Exit(libmarty.EXIT_ERR_INIT)
	}

	if (libmarty.Spawn_marty_socket(&data) != libmmp.NONE) {

		libmarty.Marty_full_log("Socket spawn failure!\n",
			libmarty.LOG_CRITICAL, &data)

		do_notify(data.VersionString, "CRITICAL: Socket spawn failure!",
			libnotify.NOTIFY_URGENCY_CRITICAL)

		os.Exit(libmarty.EXIT_ERR_SPAWN)
	}

	if (libmarty.Marty_init(&data) != libmmp.NONE) {

		libmarty.Marty_full_log("MMP initialisation failure!\n",
			libmarty.LOG_CRITICAL, &data)

		do_notify(data.VersionString, "CRITICAL: MMP initialisation failure!",
			libnotify.NOTIFY_URGENCY_CRITICAL)

		os.Exit(libmarty.EXIT_ERR_INIT)
	};

	libmarty.Marty_full_log("MMP initialisation success!\n", libmarty.LOG_STARTUP, &data)

	go libmarty.Marty_network_loop(&data)
	libmarty.Marty_full_log("Network loop started.\n", libmarty.LOG_STARTUP, &data)

	do_notify("radio", fmt.Sprintf("I have initialised with priority %d, chief.",
		data.MyParameters.Priority), libnotify.NOTIFY_URGENCY_LOW)

	for true {

		libmmp.Mmpacket_reset(&txPacket, true)
		applicationStatus = libmmp.MMPArgs[libmmp.MMP_TYPE_APPLICATION]["SUCCESS"]

		if (rxPacket != nil) {
			libmmp.Mmpacket_reset(rxPacket, true)
		}

		rxPacket = nil

		select {
			case deliveryHandle = <- data.DeliveryChannel:

				if (deliveryHandle == nil) {

					do_notify("radio", fmt.Sprintf("I have re-initialised with priority %d, chief.",
						data.MyParameters.Priority), libnotify.NOTIFY_URGENCY_LOW)
				}

				continue
			// listen for new SEND messages from client loop on channel
			case rxPacket = <- data.MessageChannel:
		}

		if (rxPacket == nil) {
			libmarty.Marty_full_log("Received NULL pointer from ITC channel!\n", libmarty.LOG_WARNING, &data)
			continue
		}

		endpointConversion = int(rxPacket.Payloads[libmarty.PAYL_IDX_ENDPOINT][0])

		if (endpointConversion > MaxEndpoint || endpointConversion < 0) {

			libmarty.Marty_full_log("Requested endpoint outside my range!\n", libmarty.LOG_WARNING, &data)
			applicationStatus = libmmp.MMPArgs[libmmp.MMP_TYPE_APPLICATION]["ENDPOINT"]

			goto JMP_TX_STATUS
		}

		endpoint = &Endpoints[endpointConversion]

		if (len(rxPacket.Payloads) < endpoint.Min) {
			libmarty.Marty_full_log("Request payload count less than expected!\n", libmarty.LOG_WARNING, &data)

			applicationStatus = libmmp.MMPArgs[libmmp.MMP_TYPE_APPLICATION]["TOO_FEW"]

			goto JMP_TX_STATUS
		}

		if (len(rxPacket.Payloads) > endpoint.Max) {
			libmarty.Marty_full_log("Request payload count more than expected!\n", libmarty.LOG_WARNING, &data)

			applicationStatus = libmmp.MMPArgs[libmmp.MMP_TYPE_APPLICATION]["TOO_MANY"]

			goto JMP_TX_STATUS
		}

		titleIndex = 3
		messageIndex = 5

		if (len(rxPacket.Payloads) == 7) {

			titleIndex = 5
			messageIndex = 6
		}

		title = libmmp.Payload_2_str(rxPacket.Payloads[titleIndex])
		message = libmmp.Payload_2_str(rxPacket.Payloads[messageIndex])

		do_notify(title, message, NOTIFICATION_MAP[endpointConversion])

		JMP_TX_STATUS:

		libmarty.Marty_set_application(&txPacket, &status, rxPacket.Payloads[0], applicationStatus)
		if (status.Error != libmmp.NONE) {
			libmarty.Marty_full_log("Couldn't set autopacket!\n", libmarty.LOG_ERROR, &data)
			continue
		}

		libmmp.Serialize(&txPacket, &status)
		if (status.Error != libmmp.NONE) {
			libmarty.Marty_full_log("Couldn't serialize autopacket!\n", libmarty.LOG_ERROR, &data)
			fmt.Printf("%d\n", status.Error)
			continue
		}

		libmarty.Marty_net_write(&txPacket, &status, &data)
	}
}
